
#ifdef WIN32
#include <windows.h>
#endif

#include <stdlib.h>
#include <math.h>

#include <GL/gl.h>

#ifndef M_PI
#define M_PI 3.14159
#endif

#define NUMSTARS 600

/* X,Y,Z of each star */
static GLfloat stars[NUMSTARS][3];
static int DisplayListNumber;

/* Initializes an array of stars, or points, each of which    
 * are unit length of one away from (0,0,0). It then compiles 
 * the starfield into a display list.                         
 */
void InitStars() {
  GLfloat x,y,z,t,r;
  int n;

#ifdef WIN32
  srand(10);
#else
  srandom(10);
#endif

  for (n=0; n<NUMSTARS; n++) {
    z = (GLfloat)n;
    z -= NUMSTARS/2.0;
    r = sqrt((NUMSTARS/2.0)*(NUMSTARS/2.0)-z*z);
#ifdef WIN32
    t = (GLfloat)(rand())/(GLfloat)(RAND_MAX);
#else
    t = (GLfloat)(random())/(GLfloat)(RAND_MAX);
#endif
    t *= 2*M_PI;
    x = r * (GLfloat)cos(t);
    y = r * (GLfloat)sin(t);
    stars[n][0] = x;
    stars[n][1] = y;
    stars[n][2] = z;
  }

  DisplayListNumber = glGenLists(1);
  glNewList(DisplayListNumber, GL_COMPILE);

  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);
  glDisable(GL_CULL_FACE);

  glEnable (GL_POINT_SMOOTH);
  glEnable (GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  glColor3f(1,1,1);
  glPointSize(1);

  glBegin(GL_POINTS);
  for (n=0; n<NUMSTARS; n+=3)
    glVertex3fv(stars[n]);
  glEnd();

  glColor3f(1,.7,.7);
  glBegin(GL_POINTS);
  for (n=1; n<NUMSTARS; n+=3)
    glVertex3fv(stars[n]);
  glEnd();

  glColor3f(.6,.6,1);
  glBegin(GL_POINTS);
  for (n=2; n<NUMSTARS; n+=3)
    glVertex3fv(stars[n]);
  glEnd();

  glEndList();
}


/*
 * Draws a starfield by changing the modelview matrix   
 * to (0,0,0) but preserving the orientation.           
 * We then draw all the stars, which were precalculated 
 * to all be a unit length of 1 away.                   
 */
void DrawStars() {
  GLdouble m[16];
  int n, x;

  glPushMatrix();
  glPushAttrib(GL_ALL_ATTRIB_BITS);

  glGetDoublev(GL_MODELVIEW_MATRIX, m);
  m[12] = m[13] = m[14] = 0;
  glLoadMatrixd(m);

  glCallList(DisplayListNumber);

  glPopAttrib();
  glPopMatrix();
}

